package com.hubgames.rawcontacts;

import android.annotation.TargetApi;
import android.content.ContentProviderOperation;
import android.content.ContentResolver;
import android.content.ContentUris;
import android.content.Context;
import android.content.OperationApplicationException;
import android.database.Cursor;
import android.net.Uri;
import android.os.AsyncTask;
import android.os.Build;
import android.os.RemoteException;
import android.provider.ContactsContract;
import android.provider.ContactsContract.CommonDataKinds.Email;
import android.text.TextUtils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedHashMap;

import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry.Registrar;

/**
 * RawContactsPlugin
 */
@TargetApi(Build.VERSION_CODES.HONEYCOMB)
public class RawContactsPlugin implements MethodCallHandler {

    private final ContentResolver contentResolver;
    private final Context ctx;
    private static final String[] PROJECTION =
            {
                    ContactsContract.Data.CONTACT_ID,
                    ContactsContract.RawContacts.DISPLAY_NAME_PRIMARY,
                    ContactsContract.Data.MIMETYPE,
                    Email.TYPE,
                    Email.ADDRESS
            };

    public RawContactsPlugin(ContentResolver contentResolver, Context ctx) {
        this.contentResolver = contentResolver;
        this.ctx = ctx;
    }

    /**
     * Plugin registration.
     */
    public static void registerWith(Registrar registrar) {
        final MethodChannel channel = new MethodChannel(registrar.messenger(), "hubgames.com/raw_contacts");
        channel.setMethodCallHandler(new RawContactsPlugin(registrar.context().getContentResolver(), registrar.activity()));
    }

    @Override
    public void onMethodCall(MethodCall call, Result result) {
        switch (call.method) {
            case "getContacts":
                this.getContacts((String) call.argument("filter"), (boolean) call.argument("getImages"), result);
                break;
            case "createContact":
                this.createContact((HashMap) call.argument("newContact"), (String) call.argument("accountType"), result);
                break;
            default:
                result.notImplemented();
                break;
        }
    }

    void getContacts(String filter, boolean getImages, Result result) {
        new GetContactsTask(result, getImages).execute(new String[]{filter});
    }

    void createContact(HashMap newContactMap, String accountType, Result result) {
        Contact newContact = Contact.fromMap(newContactMap);
        if (newContact != null) {
            try {
                ArrayList<ContentProviderOperation> operations = new ArrayList<>();
                int idx = operations.size();
                operations.add(ContentProviderOperation.newInsert(ContactsContract.RawContacts.CONTENT_URI)
                        .withValue(ContactsContract.RawContacts.ACCOUNT_TYPE, accountType)
                        .withValue(ContactsContract.RawContacts.ACCOUNT_NAME, newContact.emails.get(0).value)
                        .build());

                operations.add(ContentProviderOperation.newInsert(ContactsContract.Data.CONTENT_URI)//Step2
                        .withValueBackReference(ContactsContract.RawContacts.Data.RAW_CONTACT_ID, idx)
                        .withValue(ContactsContract.RawContacts.Data.MIMETYPE, ContactsContract.CommonDataKinds.StructuredName.CONTENT_ITEM_TYPE)
                        .withValue(ContactsContract.CommonDataKinds.StructuredName.DISPLAY_NAME, newContact.displayName) // Name of the contact
                        .build());

                operations.add(ContentProviderOperation.newInsert(ContactsContract.Data.CONTENT_URI)//Step 3
                        .withValueBackReference(ContactsContract.Data.RAW_CONTACT_ID, idx)
                        .withValue(ContactsContract.RawContacts.Data.MIMETYPE, Email.CONTENT_ITEM_TYPE)
                        .withValue(Email.ADDRESS, newContact.emails.get(0).value) // Number to be added
                        .withValue(Email.TYPE, Email.TYPE_HOME)
                        .build());

                operations.add(ContentProviderOperation.newInsert(ContactsContract.Data.CONTENT_URI)//Step 4
                        .withValueBackReference(ContactsContract.Data.RAW_CONTACT_ID, idx)
                        .withValue(ContactsContract.CommonDataKinds.Photo.MIMETYPE, ContactsContract.CommonDataKinds.Photo.CONTENT_ITEM_TYPE)
                        .withValue(ContactsContract.CommonDataKinds.Photo.PHOTO, newContact.avatar) // Photo to be added
                        .build());


                ctx.getContentResolver().applyBatch(ContactsContract.AUTHORITY, operations);

                result.success(true);
            } catch (RemoteException e) {
                e.printStackTrace();
                result.error(null, "Failed to create contact, Operation Error", null);
            } catch (OperationApplicationException e) {
                e.printStackTrace();
                result.error(null, "Failed to create contact, Operation Error", null);

            }
        }

    }

    void deleteContact(HashMap contactMap, Result result) {
        Contact contact = Contact.fromMap(contactMap);
        ArrayList<ContentProviderOperation> ops = new ArrayList<>();
        ops.add(ContentProviderOperation.newDelete(ContactsContract.RawContacts.CONTENT_URI)
                .withSelection(ContactsContract.RawContacts.CONTACT_ID + "=?", new String[]{String.valueOf(contact.identifier)})
                .build());
        try {
            contentResolver.applyBatch(ContactsContract.AUTHORITY, ops);
            result.success(true);
        } catch (Exception e) {
            result.error(null, "Failed to delete Contact", null);
        }
    }

    @TargetApi(Build.VERSION_CODES.CUPCAKE)
    private class GetContactsTask extends AsyncTask<String, Void, ArrayList<HashMap>> {

        private Result getContactResult;
        private boolean getImages;

        public GetContactsTask(Result result, boolean getImages) {
            this.getContactResult = result;
            this.getImages = getImages;
        }

        protected ArrayList<HashMap> doInBackground(String... query) {
            Cursor cursor = getCursor(query[0]);
            ArrayList<Contact> contacts = getContactsFrom(cursor);
            if (getImages) {
                for (Contact c : contacts) {
                    setAvatarDataForContactIfAvailable(c);
                }
            }
            //Transform the list of contacts to a list of Map
            ArrayList<HashMap> contactMaps = new ArrayList<>();
            for (Contact c : contacts) {
                contactMaps.add(c.toMap());
            }

            return contactMaps;
        }

        protected void onPostExecute(ArrayList<HashMap> result) {
            getContactResult.success(result);
        }
    }

    @TargetApi(Build.VERSION_CODES.HONEYCOMB)
    private void setAvatarDataForContactIfAvailable(Contact contact) {
        Uri contactUri = ContentUris.withAppendedId(ContactsContract.Contacts.CONTENT_URI, Integer.parseInt(contact.identifier));
        Uri photoUri = Uri.withAppendedPath(contactUri, ContactsContract.Contacts.Photo.CONTENT_DIRECTORY);
        Cursor avatarCursor = contentResolver.query(photoUri,
                new String[]{ContactsContract.Contacts.Photo.PHOTO}, null, null, null);
        if (avatarCursor != null && avatarCursor.moveToFirst()) {
            byte[] avatar = avatarCursor.getBlob(0);
            contact.avatar = avatar;
        }
        if (avatarCursor != null) {
            avatarCursor.close();
        }
    }

    @TargetApi(Build.VERSION_CODES.HONEYCOMB)
    private Cursor getCursor(String query) {
        String selection = "";
        String[] selectionArgs = null;
        if (query != null) {
            selectionArgs = new String[]{query};
            selection = ContactsContract.RawContacts.ACCOUNT_TYPE + " = ?";
        }
        return contentResolver.query(ContactsContract.Data.CONTENT_URI, PROJECTION, selection, selectionArgs, null);
    }

    /**
     * Builds the list of contacts from the cursor
     *
     * @param cursor
     * @return the list of contacts
     */
    @TargetApi(Build.VERSION_CODES.HONEYCOMB)
    private ArrayList<Contact> getContactsFrom(Cursor cursor) {
        HashMap<String, Contact> map = new LinkedHashMap<>();
        while (cursor != null && cursor.moveToNext()) {
            int columnIndex = cursor.getColumnIndex(ContactsContract.Data.CONTACT_ID);
            String contactId = cursor.getString(columnIndex);

            if (!map.containsKey(contactId)) {
                map.put(contactId, new Contact(contactId));
            }
            Contact contact = map.get(contactId);

            String mimeType = cursor.getString(cursor.getColumnIndex(ContactsContract.Data.MIMETYPE));
            contact.displayName = cursor.getString(cursor.getColumnIndex(ContactsContract.RawContacts.DISPLAY_NAME_PRIMARY));
            int type = -1;
            switch (mimeType) {
                case Email.CONTENT_ITEM_TYPE:
                    String email = cursor.getString(cursor.getColumnIndex(Email.ADDRESS));
                    type = cursor.getInt(cursor.getColumnIndex(Email.TYPE));
                    if (!TextUtils.isEmpty(email)) {
                        contact.emails.add(new KeyValuePair(KeyValuePair.getEmailLabel(type, cursor), email));
                    }
                    break;
            }
        }
        return new ArrayList<>(map.values());
    }
}