import 'dart:async';
import 'dart:typed_data';

import 'package:flutter/services.dart';

class RawContacts {
  static const MethodChannel _channel =
      const MethodChannel('hubgames.com/raw_contacts');

  static Future<Iterable<Contact>> getContacts(
      {String filter, bool getImages = true}) async {
    Map<String, dynamic> args = {"filter": filter, "getImages": getImages};

    final Iterable contacts = await _channel.invokeMethod('getContacts', args);
    return contacts.map((c) => Contact.fromMap(c));
  }

  static Future<bool> createContact(
      {String accountType, Contact newContact}) async {
    Map<String, dynamic> args = {
      "accountType": accountType,
      "newContact": Contact.toMap(newContact)
    };

    return await _channel.invokeMethod('createContact', args);
  }

  static Future<bool> deleteContact({Contact contact}) async {
    Map<String, dynamic> args = {
      "contact": Contact.toMap(contact),
    };
    return await _channel.invokeMethod('deleteContact', args);
  }
}

class Contact {
  Contact({
    this.displayName,
    this.givenName,
    this.middleName,
    this.prefix,
    this.suffix,
    this.familyName,
    this.company,
    this.jobTitle,
    this.emails,
    this.phones,
    this.postalAddresses,
    this.avatar,
  });

  String identifier,
      displayName,
      givenName,
      middleName,
      prefix,
      suffix,
      familyName,
      company,
      jobTitle;
  Iterable<KeyValuePair> emails = [];
  Iterable<KeyValuePair> phones = [];
  Iterable<PostalAddress> postalAddresses = [];
  Uint8List avatar;

  Contact.fromMap(Map m) {
    identifier = m["identifier"];
    displayName = m["displayName"];
    givenName = m["givenName"];
    middleName = m["middleName"];
    familyName = m["familyName"];
    prefix = m["prefix"];
    suffix = m["suffix"];
    company = m["company"];
    jobTitle = m["jobTitle"];
    emails = (m["emails"] as Iterable)?.map((m) => KeyValuePair.fromMap(m));
    phones = (m["phones"] as Iterable)?.map((m) => KeyValuePair.fromMap(m));
    postalAddresses = (m["postalAddresses"] as Iterable)
        ?.map((m) => PostalAddress.fromMap(m));
    avatar = m["avatar"];
  }

  static Map toMap(Contact contact) {
    var emails = [];
    for (KeyValuePair email in contact.emails ?? []) {
      emails.add(KeyValuePair.toMap(email));
    }
    var phones = [];
    for (KeyValuePair phone in contact.phones ?? []) {
      phones.add(KeyValuePair.toMap(phone));
    }
    var postalAddresses = [];
    for (PostalAddress address in contact.postalAddresses ?? []) {
      postalAddresses.add(PostalAddress.toMap(address));
    }
    return {
      "identifier": contact.identifier,
      "displayName": contact.displayName,
      "givenName": contact.givenName,
      "middleName": contact.middleName,
      "familyName": contact.familyName,
      "prefix": contact.prefix,
      "suffix": contact.suffix,
      "company": contact.company,
      "jobTitle": contact.jobTitle,
      "emails": emails,
      "phones": phones,
      "postalAddresses": postalAddresses,
      "avatar": contact.avatar
    };
  }
}

class PostalAddress {
  PostalAddress({
    this.label,
    this.street,
    this.city,
    this.postcode,
    this.region,
    this.country,
  });

  String label, street, city, postcode, region, country;

  PostalAddress.fromMap(Map m) {
    label = m["label"];
    street = m["street"];
    city = m["city"];
    postcode = m["postcode"];
    region = m["region"];
    country = m["country"];
  }

  static Map toMap(PostalAddress address) => {
        "label": address.label,
        "street": address.street,
        "city": address.city,
        "postcode": address.postcode,
        "region": address.region,
        "country": address.country
      };
}

/// Item class used for contact fields which only have a [key] and
/// a [value], such as emails and phone numbers
class KeyValuePair {
  KeyValuePair({this.key, this.value});

  String key, value;

  KeyValuePair.fromMap(Map m) {
    key = m["key"];
    value = m["value"];
  }

  static Map toMap(KeyValuePair i) => {"key": i.key, "value": i.value};
}
